import java.awt.*;

public class FillPolygon
{
  int fNumVertices;
  int fColor;
  int fZmax, fZMin;
  int fXmax, fXMin;
  int fYMax, fYMin;
  FillVertex fVertices[];
  int fScrWidth, fScrHeight;
  int fScrMidX, fScrMidY;
  Graphics fOffScrGraphics;

  public FillPolygon()
  {
  }

  public void Set(FillVertex[] v, int[] vlist, int numVert)
  {
    int j=0;
    // allocate pointers (not the objects)
    fVertices = new FillVertex[numVert];
    for (int i=0;i<numVert;i++)
      fVertices[i]=v[vlist[j++]];

    fNumVertices=numVert;
  }

  public void SetGraphics(Graphics offScrGraphics,
    int scrWidth, int scrHeight, int midX, int midY)
  {
    fScrWidth=scrWidth;
    fScrHeight=scrHeight;
    fScrMidX=midX;
    fScrMidY=midY;
    fOffScrGraphics=offScrGraphics;
  }

  public boolean Backface()
  {
    int p0x=fVertices[0].fSx;
    int p0y=fVertices[0].fSy;

    int p1x=fVertices[1].fSx;
    int p1y=fVertices[1].fSy;

    int p2x=fVertices[2].fSx;
    int p2y=fVertices[2].fSy;

    int z=(p1x-p0x)*(p2y-p0y)-(p1y-p0y)*(p2x-p0x);

    return (z>=0);
  }

  public void Draw()
  {
    int YDiff1, YDiff2;
    int XDiff1, XDiff2;
    int Length;
    int Err1, Err2;
    int Count1, Count2;
    int XStep1, XStep2;
    int LineX1, LineX2, LineY1, LineY2;
    int XStart, XEnd;

    // how many edges are there
    int EdgeCount=fNumVertices-1;

    // determine which vertex is at top of polygon
    // start by assuming vertex 0 is at top
    int FirstVertex=0;
    int MinY=fVertices[0].fSy;

    // search through all vertices
    for (int i=1;i<fNumVertices; i++)
    {
      // is next vertex higher
      if (fVertices[i].fSy < MinY)
      {
        FirstVertex=i;
        MinY=fVertices[i].fSy;
      }
    }

    int StartVertex1=FirstVertex;
    int StartVertex2=FirstVertex;
    int XStart1=fVertices[StartVertex1].fSx;
    int YStart1=fVertices[StartVertex1].fSy;
    int XStart2=fVertices[StartVertex2].fSx;
    int YStart2=fVertices[StartVertex2].fSy;

    // get end of edge 1 and check for wrap at last vertex
    int EndVertex1=StartVertex1-1;
    if (EndVertex1<0)
      EndVertex1=fNumVertices-1;
    int XEnd1=fVertices[EndVertex1].fSx;
    int YEnd1=fVertices[EndVertex1].fSy;

    // get end of edge 2 and check for wrap
    int EndVertex2=StartVertex2+1;
    if (EndVertex2==fNumVertices)
      EndVertex2=0;
    int XEnd2=fVertices[EndVertex2].fSx;
    int YEnd2=fVertices[EndVertex2].fSy;

    // draw the polygon
    // while there is more edge to draw
    while (EdgeCount>0)
    {
      Err1=0;
      Err2=0;

      // get abs value of y
      if ((YDiff1=YEnd1-YStart1)<0)
        YDiff1=-YDiff1;
      if ((YDiff2=YEnd2-YStart2)<0)
        YDiff2=-YDiff2;

      // get abs of x
      if ((XDiff1=XEnd1-XStart1)<0)
      {
        XStep1=-1;
        XDiff1=-XDiff1;
      }
      else
        XStep1=1;
      if ((XDiff2=XEnd2-XStart2)<0)
      {
        XStep2=-1;
        XDiff2=-XDiff2;
      }
      else
        XStep2=1;

      // QUADRANT SEPARATIONS
      // chose which of 4 routines to use

      // edge 1 slope<1 (ie: left edge approaching horizontal line)
      if (XDiff1>YDiff1)
      {
        // edge 2 slope <1 (ie: right edge is approaching horizontal)
        // LEFT EDGE APPROACH HORIZONTAL
        // RIGHT EDGE APPROACH HORIZONTAL
        if (XDiff2>YDiff2)
        {
          // the width of left edge
          Count1=XDiff1;
          // the width of right edge
          Count2=XDiff2;
          // while both edges are not completely drawn
          while (Count1>0 && Count2>0)
          {
            // drawing left edge
            // while not the time to move in y direction
            while ((Err1<XDiff1)&&(Count1>0))
            {
              // draw edge
              if (Count1-->0)
              {
                XStart1+=XStep1;
              }
              Err1+=YDiff1;
            }
            // time to move left edge in y direction (add y)
            Err1-=XDiff1;

            // drawing right edge
            // while not the time to move in y direction
            while ((Err2<XDiff2)&&(Count2>0))
            {
              // draw edge
              if (Count2-->0)
              {
                XStart2+=XStep2;
              }
              Err2+=YDiff2;
            }
            Err2-=XDiff2;

            if (XStart2>XStart1)
            {
              LineX1=XStart2;
              LineX2=XStart1;
            }
            else
            {
              LineX1=XStart1;
              LineX2=XStart2;
            }

            // draw the line
            fOffScrGraphics.drawLine(fScrMidX+LineX1,fScrMidY+YStart1,
              fScrMidX+LineX2,fScrMidY+YStart1);

            YStart1++;
            YStart2++;
          }
        }

        // LEFT EDGE APPROACH HORIZONTAL
        // RIGHT EDGE APPROACH VERTICAL
        // edge2 slope>=1
        else
        {
          Count1=XDiff1;
          Count2=YDiff2;

          while (Count1>0 && Count2>0)
          {
            while ((Err1<XDiff1)&&(Count1>0))
            {
              if (Count1-->0)
              {
                XStart1+=XStep1;
              }
              Err1+=YDiff1;
            }
            Err1-=XDiff1;

            Err2+=XDiff2;
            if (Err2>=YDiff2)
            {
              Err2-=YDiff2;
              XStart2+=XStep2;
            }
            Count2--;

            if (XStart2>XStart1)
            {
              LineX1=XStart2;
              LineX2=XStart1;
            }
            else
            {
              LineX1=XStart1;
              LineX2=XStart2;
            }

            // draw the line
            fOffScrGraphics.drawLine(fScrMidX+LineX1,fScrMidY+YStart1,
              fScrMidX+LineX2,fScrMidY+YStart1);

            YStart1++;
            YStart2++;
          }
        }
      }

      // edge 1 slope >=1 (approaching vertical line)
      else
      {
        // edge 2 slope opproach horizontal
        if (XDiff2>YDiff2)
        {
          Count1=YDiff1;
          Count2=XDiff2;

          while (Count1>0 && Count2>0)
          {
            Err1+=XDiff1;
            if (Err1>=YDiff1)
            {
              Err1-=YDiff1;
              XStart1+=XStep1;
            }
            Count1--;

            while ((Err2<XDiff2)&&(Count2>0))
            {
              if (Count2-->0)
              {
                XStart2+=XStep2;
              }
              Err2+=YDiff2;
            }
            Err2-=XDiff2;

            if (XStart2>XStart1)
            {
              LineX1=XStart2;
              LineX2=XStart1;
            }
            else
            {
              LineX1=XStart1;
              LineX2=XStart2;
            }

            // draw the line
            fOffScrGraphics.drawLine(fScrMidX+LineX1,fScrMidY+YStart1,
              fScrMidX+LineX2,fScrMidY+YStart1);

            YStart1++;
            YStart2++;
          }
        }

        // edge 2 slope approach vertical
        else
        {
          Count1=YDiff1;
          Count2=YDiff2;

          while (Count1>0 && Count2>0)
          {
            Err1+=XDiff1;
            if (Err1>=YDiff1)
            {
              Err1-=YDiff1;
              XStart1+=XStep1;
            }
            Count1--;

            Err2+=XDiff2;
            if (Err2>=YDiff2)
            {
              Err2-=YDiff2;
              XStart2+=XStep2;
            }
            Count2--;

            if (XStart2>XStart1)
            {
              LineX1=XStart2;
              LineX2=XStart1;
            }
            else
            {
              LineX1=XStart1;
              LineX2=XStart2;
            }

            // draw the line
            fOffScrGraphics.drawLine(fScrMidX+LineX1,fScrMidY+YStart1,
              fScrMidX+LineX2,fScrMidY+YStart1);

            YStart1++;
            YStart2++;
          }
        }
      }

      if (Count1<=0)
      {
        EdgeCount--;
        StartVertex1=EndVertex1;
        EndVertex1--;

        if (EndVertex1<0)
          EndVertex1=fNumVertices-1;

        XEnd1=fVertices[EndVertex1].fSx;
        YEnd1=fVertices[EndVertex1].fSy;
      }

      if (Count2<=0)
      {
        EdgeCount--;
        StartVertex2=EndVertex2;
        EndVertex2++;

        if (EndVertex2==fNumVertices)
          EndVertex2=0;

        XEnd2=fVertices[EndVertex2].fSx;
        YEnd2=fVertices[EndVertex2].fSy;
      }
    }
  }
}